<?php
  /**
  *
  * ULTIMATE Seo Urls (version 3.0.1)
  *
  * @license http://www.opensource.org/licenses/gpl-2.0.php GNU Public License
  * Copyright 2020 JMC
  * Portions Copyright 2008-2009 FWR Media, 2005 Bobby Easland
  * @Id uri_modules.php              
  */

  /**
  * Abstract class extended by all uri modules
  *
  * Sets the contract for all uri modules to implement the abstract methods of this class
  */
  abstract class aUri_Modules {
    /**
    * Abstract method implemented in the uri modules.
    *
    * Creates a link string based on the uri type
    *
    * @see includes/usu_general_functions usu_multi_language()
    * @param string $page - base filename
    * @param string $text - uri link text
    * @param string $seperator - e.g. -p- or -c-
    * @param string $value - either an integer or a path based string
    *
    * @access public
    */
    abstract public function createLinkString( $page, $text, $seperator, $value);
    /**
    * Abstract method implemented in the uri modules.
    *
    * Seperates the uri text with e.g. hyphen
    *
    * @param array $array
    * @todo method should be separateUriText( array $array = array() )
    *
    * @access public
    */
    abstract public function separateUriText( $array );
    /**
    * Abstract method implemented in the uri modules.
    * Examines the current request to determine if it is valid for a particular uri module
    *
    * @access protected
    */
    abstract protected function isValidUri();

    /**
    * Remove the language from the request uri if present
    *
    * @uses preg_match()
    * @uses array_key_exists()
    * @uses str_replace()
    *
    * @access protected
    * @return string - uri with language removed
    *
    */

    protected function withoutLanguage() {
      preg_match( '@^[a-z]{2}/@', usu_Main::i()->getVar( 'request_uri' ), $matches );
      if ( array_key_exists( 0, $matches ) ) {
        return str_replace( $matches[0], '', usu_Main::i()->getVar( 'request_uri' ) );
      }
      return usu_Main::i()->getVar( 'request_uri' );

    } // end method

  } // end class