<?php
  /**
  *
  * ULTIMATE Seo Urls (version 3.0.1)
  *
  * @license http://www.opensource.org/licenses/gpl-2.0.php GNU Public License
  * Copyright 2020 JMC
  * Portions Copyright 2008-2009 FWR Media, 2005 Bobby Easland
  * @Id mysql.php                    
  */

  /**
  * Cache system using a MySQL database
  *
  */
  final class Mysql_Cache_Module implements iCache_System {

    private static $_singleton = null;
    private static $cache_name;
    private static $cache_on;
    public $extract_query = "SELECT * FROM usu_cache WHERE cache_name = ':cache_name'";
    public $delete_query = "DELETE FROM usu_cache WHERE cache_name = ':cache_name'";
    private $insert_query = "INSERT INTO usu_cache (cache_name, cache_data, cache_date) VALUES (':cache_name', ':cache_data', ':cache_date')";
    private $insert = false;
    /**
    * Class constructor
    * @access private
    */
    private function __construct() {
    } // end constructor
    /**
    * Returns a singleton instance of the class
    *
    * Sets the cache name and checks that the cache directory is writeable
    * @uses defined()
    * @uses substr()
    * @uses md5()
    *
    * @access public
    * @return Mysql_Cache_Module
    */
    public static function i() {
      if ( !self::$_singleton instanceof Mysql_Cache_Module ) {
        if ( usu_Main::i()->getVar( 'page_modules', substr( usu_Main::i()->getVar( 'filename' ), 0, -4 ) ) instanceof aPage_Modules ) {
          self::$cache_on = ( defined( 'USU_CACHE_ON' ) && ( USU_CACHE_ON == 'True' ) ) ? true : false;
          self::$cache_name = md5( usu_Main::i()->getVar( 'page_modules', substr( usu_Main::i()->getVar( 'filename' ), 0, -4 ) )
                                           ->buildCacheName() );
          usu_Main::i()->setVar( 'cache_name', self::$cache_name );
        } else { // No module so we set the cache name as the language id plus the called file
          self::$cache_name = usu_Main::i()->getVar( 'languages_id' ) . '_' . substr( usu_Main::i()->getVar( 'filename' ), 0, -4 );
          usu_Main::i()->setVar( 'cache_name', self::$cache_name );
        }
        self::createTable();
        self::$_singleton = new self;
      }
      return self::$_singleton;
    } // end method

    /**
    * Create the initial table and fields for mySQL
    * @access private
    */
    private static function createTable() {
      $query = "CREATE TABLE IF NOT EXISTS usu_cache ( cache_name varchar(64) NOT NULL, cache_data mediumtext NOT NULL, cache_date datetime NOT NULL, UNIQUE KEY cache_name (cache_name) );";
      tep_db_query( $query );
    }


    /**
    * Stores the current cache on the destruction of the usu_Main class
    *
    * @see usu_Main::__destruct()
    * @uses serialize()
    * @uses base64_encode()
    * @uses gzdeflate()
    * @uses str_replace()
    * @uses date()
    * @param array $registry_vars - array of data to cache
    *
    * @access public
    * @return void
    */

    public function store( array $registry_vars = array() ) {
      tep_db_connect();
      if ( false !== self::$cache_on  ) {
        if ( false !== $this->insert ) {
          $data = serialize( $registry_vars ); // Serialize the registry of data
          $rawdata = base64_encode( gzdeflate( $data ) ); // encode and deflate
          $targets = array( ':cache_name', ':cache_data', ':cache_date' );
          $replacements = array( tep_db_input( self::$cache_name ), tep_db_input( $rawdata ), date( "Y-m-d H:i:s" ) );
          $query = str_replace( $targets, $replacements, $this->insert_query );
          usu_Main::i()->query( $query );
        }
      }
    } // end method
    /**
    * Retrieve the cached data
    *
    * If $insert becomes bool true then we insert data when storing, bool false we don't save as the cache already exists
    *
    * @see usu_Main::extractCacheData()
    *
    * @access public
    * @return void
    */
    public function retrieve() {
      if ( false !== self::$cache_on  ) {
        $this->insert = usu_Main::i()->extractCacheData( self::$cache_name, 'mysql', $this );
      }
    } // end method
    /**
    * Cache garbage clearance
    *
    * @param bool $file_info
    *
    * @access public
    * @return void
    */
    public function gc(  $file_info = false ) {
      $this->insert = true;
      usu_Main::i()->query( 'TRUNCATE `usu_cache`' );
    } // end method

  } // end class