<?php
  /**
  *
  * ULTIMATE Seo Urls (version 3.0.1)
  *
  *
  * @package USU_PRO
  * @license http://www.opensource.org/licenses/gpl-2.0.php GNU Public License
  * Copyright 2020 JMC
  * Portions Copyright 2008-2009 FWR Media, 2005 Bobby Easland
  * @Id mysql.php
  * @Id $Id:: sqlite.php
  */

  /**
  * Cache system using a SQLite database
  *
  * @package USU_PRO
  */
  final class Sqlite_Cache_Module implements iCache_System {

    private static $_singleton = null;
    private static $sqlite_db_file = null;
    private static $cache_name;
    private static $cache_on;
    public static $db;
    public $extract_query = "SELECT * FROM usu_cache WHERE cache_name = ':cache_name'";
    private $insert_query = "INSERT INTO usu_cache (cache_name, cache_data, cache_date) VALUES (':cache_name', ':cache_data', ':cache_date')";
    private $insert = false;
    /**
    * Class constructor
    * @access private
    */
    private function __construct() {
    } // end constructor
    /**
    * Returns a singleton instance of the class
    *
    * Sets the cache name and checks that the cache directory is writeable
    * @uses defined()
    * @uses substr()
    * @uses md5()
    * @uses is_readable()
    * @uses trigger_error()
    *
    * @access public
    * @throws - triggers an error of type E_USER_WARNING if a new database cannot be created
    * @return Sqlite_Cache_Module
    */
    public static function i() {
      if ( !self::$_singleton instanceof Sqlite_Cache_Module ) {
        if ( usu_Main::i()->getVar( 'page_modules', substr( usu_Main::i()->getVar( 'filename' ), 0, -4 ) ) instanceof aPage_Modules ) {
          self::$cache_on = ( defined( 'USU_CACHE_ON' ) && ( USU_CACHE_ON == 'True' ) ) ? true : false;
          self::$cache_name = md5( usu_Main::i()->getVar( 'page_modules', substr( usu_Main::i()
                                           ->getVar( 'filename' ), 0, -4 ) )
                                           ->buildCacheName() );
          usu_Main::i()->setVar( 'cache_name', self::$cache_name );
        } else { // No module so we set the cache name as the language id plus the called file
          self::$cache_name = usu_Main::i()->getVar( 'languages_id' ) . '_' . substr( usu_Main::i()->getVar( 'filename' ), 0, -4 );
        }
        self::$sqlite_db_file = usu_Main::i()->getVar( 'cache_system_path' ) . 'sqlite/usu_cache.db';
        self::createDatabase();
        self::$_singleton = new self;
      }
      return self::$_singleton;
    } // end method
    /**
    * Returns a limited singleton instance of the class specifically for admin
    *
    * Allows admin to access a limited version of the class in order to truncate the database
    *
    * @access public
    * @return Sqlite_Cache_Module
    */
    public static function admini() {
      if ( !self::$_singleton instanceof Sqlite_Cache_Module ) {
        self::$sqlite_db_file = realpath( dirname( __FILE__ ) ) . '/sqlite/usu_cache.db';
        self::createDatabase();
        self::$_singleton = new self;
      }
      return self::$_singleton;
    }
    /**
    * Create the SQLite database if it doesn't already exist
    *
    * @uses is_readable()
    * @uses trigger_error()
    *
    * @access protected
    * @throws - triggers an error of type E_USER_WARNING if a new database cannot be created
    * @return void
    */
    protected static function createDatabase() {
      if ( !is_readable( self::$sqlite_db_file ) ) {
        try {
          self::$db = new SQLite3( self::$sqlite_db_file );
        }
        catch (Exception $exception) {
          trigger_error( 'Failed: ' . $exception->getMessage(), E_USER_WARNING );
        }
        self::createTables();
      } else {
        try {
          self::$db = new SQLite3( self::$sqlite_db_file );
        }
        catch (Exception $exception) {
          trigger_error( 'Failed: ' . $exception->getMessage(), E_USER_WARNING );
        }
      }
    }
    /**
    * Stores the current cache on the destruction of the usu_Main class
    *
    * @see usu_Main::__destruct()
    * @uses serialize()
    * @uses base64_encode()
    * @uses gzdeflate()
    * @uses str_replace()
    * @uses date()
    * @uses sqlite_escape_string()
    * @param array $registry_vars - array of data to cache
    *
    * @access public
    * @return void
    */
    public function store( array $registry_vars = array() ) {
      if ( false !== self::$cache_on  ) {
        if ( false !== $this->insert ) {
          $data = serialize( $registry_vars ); // Serialize the registry of data
          $rawdata = base64_encode( gzdeflate( $data ) ); // encode and deflate
          $targets = array( ':cache_name', ':cache_data', ':cache_date' );
          $replacements = array( self::$db->escapeString( self::$cache_name ), self::$db->escapeString( $rawdata ), date( "Y-m-d H:i:s" ) );
          $query = str_replace( $targets, $replacements, $this->insert_query );
          self::$db->query( $query );
        }
      }
    }
    /**
    * Retrieve the cached data
    *
    * If $insert becomes bool true then we insert data when storing, bool false we don't save as the cache already exists
    *
    * @see usu_Main::extractCacheData()
    *
    * @access public
    * @return void
    */
    public function retrieve() {
      if ( false !== self::$cache_on  ) {
        $this->insert = usu_Main::i()->extractCacheData( self::$cache_name, 'sqlite', $this );
      }
    }
    /**
    * Cache garbage clearance
    *
    * @uses file_exists()
    * @uses unlink()
    * @param bool $file_info
    *
    * @access public
    * @return void
    */
    public function gc( $file_info = false ) {
      if ( file_exists( self::$sqlite_db_file ) ) {
        self::$db->query( 'DELETE FROM usu_cache' );
        self::$db->query( 'VACUUM' );
      }
    }
    /**
    * Retrieve an instance of SQLite3
    *
    * @access public
    * @return SQLite3
    */
    public function getDb() {
      return self::$db;
    }
    /**
    * Create the initial table and fileds for SQLite3
    * @access private
    */
    private static function createTables() {
      $create_query = "
      CREATE TABLE usu_cache (
        cache_name,
        cache_data,
        cache_date
      );

      CREATE UNIQUE INDEX idx_cache_name ON usu_cache( cache_name );";

      self::$db->query( $create_query );
    }

  } // end class
