<?php
  /**
  *
  * ULTIMATE Seo Urls (version 3.0.1)
  *
  * @license http://www.opensource.org/licenses/gpl-2.0.php GNU Public License
  * Copyright 2020 JMC
  * Portions Copyright 2008-2009 FWR Media, 2005 Bobby Easland
  * @Id hreflang.php
  */


  /**
  * Main hreflang class
  *
  */
  class hreflang {
    private static $_singleton = null;

    protected $page;
    protected $mode;
    protected $get;
    protected $lng;
    protected $language;
    protected $session_started;
    protected $language_code_array = array();
    protected $lang_array;

    public function __construct () {}

    /**
    * Return a singleton instance of the class
    *
    * @access public
    * @return hreflang
    */
    public static function i() {
      if ( !self::$_singleton instanceof hreflang ) {
        self::$_singleton = new self;
      }
      return self::$_singleton;
    } // end method

    /**
    * Actioned on includes/application_top.php
    *
    * After the variables have all been set this is were USU is actually initiated
    *
    * @param mixed $lng - instance of language / empty array
    * @param int $languages_id - passed by reference as USU may reset this value in multi language mode
    * @param string $language - passed by reference as USU may reset this value in multi language mode
    * @param $force_language - force links to be produced for a particular language
    * @see usu_Bootstrap::bootStrapper()
    *
    * @access public
    * @return void
    */
    public function initiate($navigation_data) {
      global $lng;
      $navigation_data_array = array_pop( $navigation_data );
      $this->page = $navigation_data_array['page'];
      $this->get =  $navigation_data_array['get'];
      $this->setLanguage( $lng );
      $this->language = $GLOBALS['language'];
      $this->session_started = $GLOBALS['session_started'];
      $this->getLanguageSelectionHtml();

    } // end method

    // Returns an array containing data of all alternate languages (other than current language)
    public function lang_array() {
      return $this->lang_array;
    }

    protected function setLanguage( $lng ) {
      if ( ( false === $lng ) || ( !$lng instanceof language ) ) {
        $this->lng = new language();
        $this->setLanguageArray();
        return $this->lng;
      }
      $this->lng = $lng;
      $this->setLanguageArray();
    }

    protected function setLanguageArray() {
      foreach ( $this->lng->catalog_languages as $language_code => $unused ) {
        $this->language_code_array[] = $language_code;
      }
    }

    protected function getLanguageSelectionHtml() {
      foreach ( $this->lng->catalog_languages as $language_code => $language_data ) {
        // Sets USU language to the current selection
        usu_Main::i()->initiate( array(), $language_data['id'],  $language_data['directory'], true );
      
        // Loads character conversion for current language
        $character_conv_filepath = usu_Main::i()->getVar( 'includes_path' ) . 'character_conversion/' . usu_Main::i()->getVar( 'language' ) . '.php';
        if ( is_readable( $character_conv_filepath ) ) {
          include_once $character_conv_filepath;
          if ( isset( $char_convert ) && is_array( $char_convert ) && !empty( $char_convert ) ) {
            usu_Main::i()->setVar( 'character_conversion', $char_convert );
          }
        } else {
          usu_Main::i()->setVar( 'character_conversion', '' );
        }

        $this_link = $this->buildLink( $default = ( $language_code === DEFAULT_LANGUAGE ), $language_code );

        $lang_array[$language_data['id']]['code'] = $language_code;
        $lang_array[$language_data['id']]['id'] = $language_data['id'];
        $lang_array[$language_data['id']]['name'] = $language_data['name'];
        $lang_array[$language_data['id']]['image'] = $language_data['image'];
        $lang_array[$language_data['id']]['directory'] = $language_data['directory'];
        $lang_array[$language_data['id']]['href'] = $this_link ;
      }
      // Sets the languages array for modules:
      $this->lang_array = $lang_array;

      // Resets USU language to the current language for the rest of the links on the page
      usu_Main::i()->initiate( array(),  $_SESSION['languages_id'], $_SESSION['language'], true );
    }

    protected function buildLink($default = false, $language_code ) {
      if ( false === $this->session_started ) {
        return  $this->multiLanguage( tep_href_link( 'index.php' ), $default, $language_code );
      }
      return $this->multiLanguage( tep_href_link( $this->page, $this->getQuery() ), $default, $language_code );
    }

    protected function getQuery() {
      if ( tep_not_null( $this->get ) ) {
        if ( array_key_exists( tep_session_name(), $this->get ) ) {
          unset( $this->get[tep_session_name()] );
        }
        return http_build_query( $this->get );
      }
      return '';
    }

    protected function removeQueryString( $target ) {
      if ( false === strpos( $target, '?' ) ) {
        return $target;
      }
      return substr( $target, 0, strpos( $target, '?' ) );
    }

    protected function removeLanguageMarkers( $target, $cookie_path ) {
      $return = preg_replace( '@' . $cookie_path . '(' . implode( '|', $this->language_code_array ) . ')/@', $cookie_path, $target );
      $return = preg_replace( '@' . $this->page . '/(' . implode( '|', $this->language_code_array ) . ')@', $this->page, $return );
      return $return;
    }

    protected function multiLanguage( $link, $default, $language_code ) {
      $server = HTTP_SERVER . DIR_WS_CATALOG;
      $link = $this->removeLanguageMarkers( $link, DIR_WS_CATALOG );
      if ( false !== $default ) {
        $no_querystring = $this->removeQueryString( $link );
        // Is the page index.php and after the querystring is removed is index.php the ending characters of the URL, if so we chop it off leaving just the domain
        if ( ( $this->page == 'index.php' ) && ( substr( $no_querystring, ( strlen( $no_querystring ) - strlen( $this->page ) ), strlen( $no_querystring ) ) == $this->page ) ) {
          return str_replace( $this->page, '', $link );
        }
        return $link;
      }
      // If the .php filename is present in the link
      if ( false !== strpos( $link, $this->page ) ) {
        return str_replace( $this-> page, $this->page . '/' . $language_code, $link );
      }
      return str_replace( $server, $server . $language_code . '/', $link );
    }

  } // end class
