<?php
  /**
  *
  * ULTIMATE Seo Urls (version 3.0.1)
  *
  * @license http://www.opensource.org/licenses/gpl-2.0.php GNU Public License
  * Copyright 2020 JMC
  * Portions Copyright 2008-2009 FWR Media, 2005 Bobby Easland
  * @Id usu_general_functions.php
  */

  /**
  * General functions used by USU PRO
  */

  /**
  * Standard osCommerce tep_href_link() link wrapper function
  * Includes changes to add language support
  * @uses die()
  * @uses substr()
  * @uses strstr()
  * @uses str_replace()
  *
  *
  * @param string $page - base file name or sometimes a path
  * @param string $parameters - querystring parameters
  * @param string $connection - SSL / NONSSL
  * @param bool $add_session_id
  * @param bool $search_engine_safe
  * @return string - URL
  */

  function tep_href_link_original($page = '', $parameters = '', $connection = null, $add_session_id = true, $search_engine_safe = true) {
    global $SID;

    $page = tep_output_string($page);

    if (!tep_not_null($page)) {
      die('<h5>Error!</h5><p>Unable to determine the page link!</p>');
    }

    if (( $page == 'index.php' ) && USU_HOME_PAGE_REDIRECT == 'True' && $parameters === '' && usu_Main::i()->getVar( 'current_language', 'code') == DEFAULT_LANGUAGE ) {
      $page ="";
    }

    $link = HTTP_SERVER . DIR_WS_CATALOG . $page;

    $multi_language_valid = true;
    if ( ( false !== strpos( $page, '/' ) ) || ( substr( $page, -4, strlen( $page ) ) != '.php' ) ) {
      $multi_language_valid = false;
    }

    // USU adds language marker:
    if (tep_not_null($parameters)) {
      $link .= ( $multi_language_valid ? usu_multi_language( 'left' ) : null ) . '?' . tep_output_string($parameters);
      $separator = '&';
    } else {
      $link .= ( $multi_language_valid ? usu_multi_language( 'left' ) : null );
      $separator = '?';
    }

    $link = rtrim($link, '&?');

// Add the session ID when moving from different HTTP and HTTPS servers, or when SID is defined
    if ( $add_session_id && $GLOBALS['session_started'] && isset($SID) && (SESSION_FORCE_COOKIE_USE == 'False') && tep_not_null($SID)) {
      $_sid = $SID;
      if (isset($_sid)) {
        $link .= $separator . tep_output_string($_sid);
      }
    }


    while (strpos($link, '&&') !== false) $link = str_replace('&&', '&', $link);

    $link = str_replace('&', '&amp;', $link);

    return $link;
  } // End function

  /**
  * Simple but powerful tool for debugging variables and arrays
  * @uses is_array()
  * @uses is_object()
  * @uses var_dump()
  * @uses print_r()
  *
  * @param mixed $output - array or a variable to be output
  * @param bool $exit - true will exit the script after output / false will output but the script continues
  * @param string $file - Tells us which file the code was initiated in. The input was __FILE__
  * @param string $line - Tells us which line in the file the code was initiated in. The input was __LINE__
  * @param string $function - function to use to output an array, the default is print_r it could also be var_dump
  */
  function usu_say( $output, $exit = false, $file = false, $line = false, $function = 'print_r' ) {
    $additional = '';
    if ( false !== $file ) {
      $additional .= 'Fine: ' . $file . '<br />' . PHP_EOL;
    }
    if ( false !== $line ) {
      $additional .= 'Line: ' . $line . '<br />' . PHP_EOL;
    }
    if ( is_array( $output ) || is_object( $output ) ) {
      switch ( $function ) {
        case 'var_dump':
          echo $additional . '<pre>' . PHP_EOL;
          echo var_dump( $output );
          echo '</pre>' . PHP_EOL;
          break;
        default:
          echo $additional . '<pre>' . print_r( $output, true ) . '</pre><br />' . PHP_EOL;
          break;
      }
    } else {
      echo $additional . $output . '<br />' . PHP_EOL;
    }
    if ( false !== $exit ) {
      exit;
    }
  } // end function
  /**
  * Standard whitelist value cleansing
  * @uses preg_replace()
  *
  * @param string $value - value to be cleansed
  * @return string - cleansed value
  */
  function usu_cleanse( $value ) {
    return preg_replace( '@[^a-z0-9_\%{}]@i', '', $value );
  }
  /**
  * Remove the querystring from a uri
  * @uses strpos()
  * @uses str_replace()
  * @uses strrchr()
  *
  * @param string $path - URI
  * @return string - stripped URI
  */
  function remove_querystring( $path ) {
    if ( false !== strpos( $path, '?' ) ) {
      return str_replace( strrchr( $path, '?' ), '', $path ); // Remove any/all querystring if it exists
    }
    return $path;
  } // End method
  /**
  * Remove the session id from a uri
  * @uses htmlspecialchars_decode()
  * @uses strpos()
  * @uses preg_match()
  * @uses ltrim()
  * @uses strrchr()
  * @uses explode()
  *
  * @param string $uri
  * @return string - uri stripped of session id
  */
  function remove_session_id( $uri ) {
    $uri = htmlspecialchars_decode( $uri );
    if ( false !== ( $pos = strpos( $uri, '?' ) ) ) {
      $session_name = tep_session_name();
      if ( preg_match( '@[\?&]' . $session_name . '=[a-z0-9]+@', $uri ) ) {
        $qs = ltrim( strrchr( $uri, '?' ), '?' );
        $uri = substr( $uri, 0, $pos );
        $qs_array = explode( '&', $qs );
        $separator = '?';
        foreach ( $qs_array as $numkey => $keypair ) {
          if ( false === strpos( $keypair, $session_name ) ) {
            $uri .= $separator . $keypair;
            $separator = '&';
          }
        }
      }
    }
    return $uri;
  }
  /**
  * Force www.mysite.com/index.php to 301 redirect to www.mysite.com/
  * @uses defined()
  * @uses substr()
  * @uses strlen()
  */
  function do_homepage_redirect( $url = false ) {
    if ( defined( 'USU_HOME_PAGE_REDIRECT' ) && ( USU_HOME_PAGE_REDIRECT == 'True' )
                                              && ( USU_ENABLED == 'True' ) ) {
      if ( false === $url ) {
        $original_request_uri = remove_querystring( usu_Main::i()->getVar( 'original_request_uri' ) );
        $possible_file = substr( $original_request_uri, ( strlen( $original_request_uri ) - strlen( 'index.php' ) ), strlen( 'index.php' ) );
        if ( !tep_not_null( usu_Main::i()->getVar( 'request_uri' ) ) && ( $possible_file == 'index.php' ) ) {
          return true;
      }
      } else {
        $url = remove_querystring( $url );
        $possible_file = substr( $url, ( strlen( $url ) - strlen( 'index.php' ) ), strlen( 'index.php' ) );
        if ( $possible_file == 'index.php' ) {
          return true;
        }
      }
    }
    return false;
  } // end method;
  /**
  * Directory iterator
  * @uses substr()
  *
  * @param string $fullpath - full path to the director we wish to iterate
  * @return array - numerical index of returned pages
  */
  function usu_dir_iterator( $fullpath ) {
    $iterator = new DirectoryIterator( $fullpath );
    while( $iterator->valid() ) {
     if ( !$iterator->isDot() && $iterator->isFile() && $iterator->isReadable() && ( substr( $iterator->getFilename(), -4, 4 ) == '.php' ) ) {
       $files[] = $iterator->getFilename();
     }
     $iterator->next();
    }
    return $files;
  } // End method
  /**
  * Adds language code to a uri
  * @uses defined()
  *
  * @param string $separator
  * @return string 2 letter language code with a / added if required
  */
  function usu_multi_language( $separator = false ) {

    // Check if more than one language is installed
    if ( !defined( 'USU_ENABLED' ) || ( USU_ENABLED != 'True' )
                                    || (count ( usu_Main::i()->getVar( 'catalog_languages' ) ) < 2  || isset( $_GET['language'] ))
                                    || ( false === usu_Main::i()->getVar( 'current_language', 'code' ) )
                                    || ( DEFAULT_LANGUAGE == usu_Main::i()->getVar( 'current_language', 'code' ) ) ) {
      return false;
    }
    switch ( true ) {
      case $separator == 'left':
        return '/' . usu_Main::i()->getVar( 'current_language', 'code' );
        break;
      case $separator == 'right':
        return usu_Main::i()->getVar( 'current_language', 'code' ) . '/';
        break;
      case $separator == 'both':
        return '/' . usu_Main::i()->getVar( 'current_language', 'code' ) . '/';
        break;
      default:
        return usu_Main::i()->getVar( 'current_language', 'code' );
        break;
    }
  } // end function
  /**
  * Standardises module naming
  * @uses str_replace()
  * @uses ucwords()
  * @uses strtolower()
  *
  * @param string $filename
  * @param string $module_suffix
  */
  function module_naming_convention( $filename, $module_suffix ) {
    return str_replace( ' ', '_', ucwords( str_replace( '_', ' ', strtolower( $filename ) ) ) ) . $module_suffix;
  }
  /**
  * Attempt to make a path writeable
  * @uses is_readable()
  * @uses is_writeable()
  * @uses function_exists()
  * @uses chmod()
  * @uses clearstatcache()
  *
  * @param string $path - path to file or directory
  */
  function usu_make_writeable( $path ) {
    if ( false === is_readable( $path ) ) {
      return false;
    }
    if ( !is_writeable( $path ) ) {
      if ( function_exists( 'chmod' ) ) {
        @chmod( $path, 0755 );
        clearstatcache();
        if ( !is_writeable( $path ) ) {
          @chmod( $path, 0777 );
          clearstatcache();
        }
      }
    }
    if ( !is_writeable( $path ) ) {
      return false;
    } else return true;
  } // end function
  /**
  * Performance reporting
  *
  * reports on links built queries used, time to load cache, cache size
  */
  function performance() {

    if ( (defined( 'USU_OUPUT_PERFORMANCE' ) && ( USU_OUPUT_PERFORMANCE !== 'True' )) || USU_ENABLED !== 'True' ) {
      return false;
    }
    $cache_loaded_colour = usu_Main::$performance['cache_loaded'] == 'True' ? 'green' : 'red';
    ( USU_CACHE_ON == 'True' ) ? $status = '<span style="color: green; font-weight: bold;">On</span>' : $status = '<span style="color: red; font-weight: bold;">Off</span>';
    ( USU_ENABLED == 'True' ) ? $status2 = '<span style="color: green; font-weight: bold;">On</span>' : $status2 = '<span style="color: red; font-weight: bold;">Off</span>';
    $total_query_time = 0;
    foreach ( usu_Main::$performance['queries'] as $index => $query ) {
      $total_query_time += $query['time'];
    }
?>
<div style="padding: 3em; font-family: verdana; width: 750px; margin-left: auto ; margin-right: auto; clear: both;">
  <div style="width: 100%; background-color: #ffffdd; border: 1px solid #1659AC; font-size: 10pt;">
    <div style="background-color: #2E8FCA; font-size: 12pt; font-weight: bold; padding: 0.5em; color: #00598E;">
      <div style="float: right; color: #0073BA; font-weight: bold; font-size: 16pt; margin-top: -0.2em;">FWR MEDIA</div>
        ULTIMATE Seo Urls <span style="color: #CC3300;">5</span> PRO <span style="font-size: xx-small; color: #dddddd; font-style: italic;"> ( <?= usu_Main::$usu_version; ?> )</span> - Performance
    </div>
    <div style="padding: 0.5em; color: #027AC6; font-size: 10pt;">Ultimate SEO Urls is <?= $status2; ?></div>
    <div style="padding: 0.5em; background-color: #CCE3F1; color: #027AC6; font-size: 10pt;">Standard URI produced: <?= usu_Main::$performance['std_urls']; ?></div>
    <div style="padding: 0.5em; color: #027AC6; font-size: 10pt;">SEO URI produced: <?= usu_Main::$performance['seo_urls']; ?></div>
    <div style="padding: 0.5em; background-color: #CCE3F1; color: #027AC6; font-size: 10pt;">Query Count: <?= usu_Main::$performance['querycount']; ?></div>
    <div style="padding: 0.5em; color: #027AC6; font-size: 10pt;">Queries Saved: <?= usu_Main::$performance['queries_saved']; ?></div>
    <div style="padding: 0.5em; background-color: #CCE3F1; color: #027AC6; font-size: 10pt;">Cache load time: <?= usu_Main::$performance['time_extracting_cache']; ?> seconds <span style=" font-size: x-small; font-style: italic;">( includes gzinflate/base64_decode/unserialize )</span></div>
    <div style="padding: 0.5em; color: #027AC6; font-size: 10pt;">Data loaded from cache: <span style="font-weight: bold; color: <?= $cache_loaded_colour; ?>;"><?= usu_Main::$performance['cache_loaded']; ?></span> <span style=" font-size: x-small; font-style: italic;">( Cache System: <?= usu_Main::$performance['cache_system']; ?> )</span></div>
    <div style="padding: 0.5em; background-color: #CCE3F1; color: #027AC6; font-size: 10pt;">Total query time: <?= $total_query_time; ?> seconds</div>
    <div style="padding: 0.5em; color: #027AC6; font-size: 10pt;">Cache system is <?= $status; ?></div>
    <div style="padding: 0.5em; background-color: #CCE3F1; color: #027AC6; font-size: 10pt;"><div style="padding: 0.5em;"><span style="font-weight: bold; text-decoration: underline;">Standard Urls:</span></div>
      <div>
<?php
      foreach ( usu_Main::$performance['std_url_array'] as $index => $url ) {
        echo '          ' . $url . '<br />' . PHP_EOL;
      }
?>
      </div>
    </div>
    <div style="padding: 0.5em; color: #027AC6; font-size: 10pt;"><div style="padding: 0.5em;"><span style="font-weight: bold; text-decoration: underline;">Seo Urls:</span></div>
      <div>
<?php
      $url_count = 0;
      foreach ( usu_Main::$performance['seo_url_array'] as $index => $url ) {
        $url_count++;
        echo "       $url_count " . $url . '<br />' . PHP_EOL;
      }
?>
      </div>
    </div>
    <div style="background-color: #fff; padding: 0.5em; color: #737373; font-size: 10pt;"><div style="padding: 0.5em;"><span style="font-weight: bold; text-decoration: underline;">Queries:</span></div>
<?php
      $queries_count = 0;

      foreach ( usu_Main::$performance['queries'] as $index => $query ) {
        $queries_count++;
        echo '      <div style="padding: 0.2em; font-family: tahoma; font-size: 7pt;"><b>Time:</b> ' . $query['time'] . " seconds<br /><b>Query: $queries_count</b> " . $query['query'] . '</div>' . PHP_EOL;
      }
?>
    </div>
  </div>
</div>
<?php
    } // end function

  function usu_show_vars( $vars ) {
    if (USU_ENABLED == 'True') {
?>
<div style="padding: 3em; font-family: verdana; margin-left: auto; margin-right: auto; clear: both;">
  <div style="width: 100%; background-color: #ffffdd; border: 1px solid #1659AC;">
    <div style="background-color: #2E8FCA; font-size: medium; font-weight: bold; padding: 0.5em; color: #00598E;">
      <div style="float: right; color: #0073BA; font-weight: bold; font-size: 18pt; margin-top: -0.2em;">FWR MEDIA</div>
      Debug: Variable &amp; Array Output. ( $this<span style="color: #000000;">=></span>vars )
    </div>
<?php
    usu_say( $vars );
?>
  </div>
</div>
<?php
    }
  } // end function

  /**
  * Check if Phoenix versions is at least $a
  *
  * returns false if the version number is smaller than Phoenix version
  * returns true if the version number is equal or bigger than Phoenix version
  */

  function phoenix_min_version ($a) {
    $a = explode(".", $a);
    $b = explode(".", tep_get_version());
    for ($i = 0; $i < 4; $i++) {
      if ($a[$i] > $b[$i]) {
        return false;
      } else if ($a[$i] < $b[$i])  {
        return true;
      }
    }
    return true;
  } // end function
