<?php
  /**
  *
  * ULTIMATE Seo Urls (version 3.0.1)
  *
  * @license http://www.opensource.org/licenses/gpl-2.0.php GNU Public License
  * Copyright 2020 JMC
  * @Id usu_installer.php
  */

  /**
  * Abstract class extended by all page modules.
  *
  * Enforces the contract where all page modules are required to have methods acquireLinkText() and buildLink()
  *
  */

  global $language;

  class usu_installer {

    private $config_group_id = null;
    private $max_sort_query = "SELECT MAX(sort_order) AS current_max FROM configuration_group";
    private $usu_version ="3.0.1";
    private $htaccess_path = DIR_FS_CATALOG . '.htaccess';


    function __construct () {
    }
    // end constructor

    function uninstall() {
      global $messageStack;
      tep_db_query ('DELETE FROM configuration
                    WHERE configuration_key LIKE "USU_%"');
      tep_db_query ('DELETE FROM hooks
                     WHERE hooks_method = "Usu_init"');
      tep_db_query('UPDATE hooks
                    SET hooks_method = "href_link"
                    WHERE hooks_method = "usu_href_link"');
      tep_db_query ('DELETE FROM configuration_group
                    WHERE configuration_group_title = "Ultimate Seo Urls"');
      tep_db_query("UPDATE configuration 
                    SET configuration_value = 'False' 
                    WHERE configuration_key = 'MODULE_NAVBAR_LANGUAGES_USU_STATUS'");
      tep_db_query("UPDATE configuration
                    SET configuration_value = 'False'
                    WHERE configuration_key = 'MODULE_BOXES_LANGUAGES_USU_STATUS'");
      tep_db_query("UPDATE configuration
                    SET configuration_value = 'False'
                    WHERE configuration_key = 'MODULE_HEADER_TAGS_HREFLANG_USU_STATUS'");

      $contents = file_get_contents( $this->htaccess_path );
      $contents = preg_replace ('/\n(## BEGIN USU URL rewrite rules ##(?s).*## END USU URL rewrite rules ##)/m', '', $contents);
      file_put_contents( $this->htaccess_path, $contents );

      $messageStack->add_session('Ultimate SEO Urls uninstalled', 'success');
      tep_redirect ('usu.php');

    }

    function upgrade () {
      global $messageStack;
      if (USU_VERSION < '3.0.1'){
      // Change htacces:
        $this->addHtaccessEntries();
      // Change database:
        // Change version number:
        tep_db_query ('UPDATE configuration SET configuration_value = "' . $this->usu_version . '" WHERE configuration_key = "USU_VERSION"');
      // Remove entry
        tep_db_query ('DELETE FROM configuration WHERE configuration_key = "USU_INFO_PAGES_URI"');
      // add messagestack:
        $messageStack->add_session(sprintf(WARNING_USU_UPGRADED, $this->usu_version), 'success');
      }
    }

    function check () {
      global $messageStack;
      if (version_compare(USU_VERSION, $this->usu_version) == -1){
        $this->upgrade();
        tep_redirect(tep_href_link('usu.php'));
      }
    } // end method

    function install () {
      global $messageStack;

      // Add db entries:
      $this->config_group_id = $this->getMaxSort();
      $this->addConfigGroup();
      $this->addConfigSettings();
      $this->addHooks();
      $this->addHtaccessEntries();
      $messageStack->add_session('Ultimate SEO Urls ' . $this->usu_version . ' installed', 'success');
    } // end method

    function addHooks() {
      // Update hooks for tep_href_link function and class initialization:
      tep_db_query("UPDATE hooks
                    SET hooks_method = 'usu_href_link'
                    WHERE hooks_method = 'href_link'");

      tep_db_query("INSERT INTO hooks (hooks_site, hooks_group, hooks_action, hooks_code, hooks_class, hooks_method)
                    VALUES ('shop', 'system', 'startApplication', '_16_usu_init', 'Usu_init', 'i');");

    } // end method

    function addConfigGroup() {
      tep_db_query ("INSERT INTO configuration_group (configuration_group_id, configuration_group_title, configuration_group_description, sort_order, visible)
                VALUES ('" . $this->config_group_id . "', 'Ultimate Seo Urls', 'Options for ULTIMATE Seo Urls', '" . $this->config_group_id . "', '0');");
    } // end method

    function setConfigSettings () {
      if ( !class_exists( 'SQLite3' ) ) {
        $standard_cache_strategy = 'file';
      } else {
        $standard_cache_strategy = 'sqlite';
      }

      $USU_configs = [array('configuration_title'       => TITLE_USU_VERSION,
                             'configuration_key'         => 'USU_VERSION',
                             'configuration_value'       => $this->usu_version,
                             'configuration_description' => TEXT_USU_VERSION,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 1,
                             'date_added'                => 'now()'
                            ),
                      array ('configuration_title'       => TITLE_USU_ENABLED,
                             'configuration_key'         => 'USU_ENABLED',
                             'configuration_value'       => 'True',
                             'configuration_description' => TEXT_USU_ENABLED,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 2,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['True', 'False'], ",
                            ),
                      array ('configuration_title'       => TITLE_USU_CACHE_ON,
                             'configuration_key'         => 'USU_CACHE_ON',
                             'configuration_value'       => 'True',
                             'configuration_description' => TEXT_USU_CACHE_ON,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 3,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['True', 'False'], "                          ),                     array ('configuration_title'       => TITLE_USU_CACHE_SYSTEM,
                             'configuration_key'         => 'USU_CACHE_SYSTEM',
                             'configuration_value'       => $standard_cache_strategy,
                             'configuration_description' => TEXT_USU_CACHE_SYSTEM,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 4,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['mysql', 'file','sqlite'], "
                            ),
                      array ('configuration_title'       => TITLE_USU_CACHE_DAYS,
                             'configuration_key'         => 'USU_CACHE_DAYS',
                             'configuration_value'       => 7,
                             'configuration_description' => TEXT_USU_CACHE_DAYS,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 5,
                             'date_added'                => 'now()'
                              ),
                      array ('configuration_title'       => TITLE_USU_URLS_TYPE,
                             'configuration_key'         => 'USU_URLS_TYPE',
                             'configuration_value'       => 'path_rewrite',
                             'configuration_description' => TEXT_USU_URLS_TYPE,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 6,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['standard','path_standard','rewrite','path_rewrite'], "
                            ),
                      array ('configuration_title'       => TITLE_USU_PRODUCTS_LINK_TEXT_ORDER,
                             'configuration_key'         => 'USU_PRODUCTS_LINK_TEXT_ORDER',
                             'configuration_value'       => 'p',
                             'configuration_description' => TEXT_USU_PRODUCTS_LINK_TEXT_ORDER,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 7,
                             'date_added'                => 'now()'
                            ),
                      array ('configuration_title'       => TITLE_USU_FILTER_SHORT_WORDS,
                             'configuration_key'         => 'USU_FILTER_SHORT_WORDS',
                             'configuration_value'       => '2',
                             'configuration_description' => TEXT_USU_FILTER_SHORT_WORDS,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 8,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['0','1','2','3'], "
                            ),
                      array ('configuration_title'       => TITLE_USU_ADD_CAT_PARENT,
                             'configuration_key'         => 'USU_ADD_CAT_PARENT',
                             'configuration_value'       => 'True',
                             'configuration_description' => TEXT_USU_ADD_CAT_PARENT,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 9,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['True', 'False'], "
                            ),
                      array ('configuration_title'       => TITLE_USU_REMOVE_ALL_SPEC_CHARS,
                             'configuration_key'         => 'USU_REMOVE_ALL_SPEC_CHARS',
                             'configuration_value'       => 'True',
                             'configuration_description' => TEXT_USU_REMOVE_ALL_SPEC_CHARS,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 10,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['True', 'False'], "
                            ),
                      array ('configuration_title'       => TITLE_USU_HOME_PAGE_REDIRECT,
                             'configuration_key'         => 'USU_HOME_PAGE_REDIRECT',
                             'configuration_value'       => "False",
                             'configuration_description' => TEXT_USU_HOME_PAGE_REDIRECT,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 11,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['True', 'False'], "
                            ),

                      array ('configuration_title'       => TITLE_USU_OUPUT_PERFORMANCE,
                             'configuration_key'         => 'USU_OUPUT_PERFORMANCE',
                             'configuration_value'       => "False",
                             'configuration_description' => TEXT_USU_OUPUT_PERFORMANCE,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 12,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['True', 'False'], "
                            ),

                      array ('configuration_title'       => TITLE_USU_DEBUG_OUPUT_VARS,
                             'configuration_key'         => 'USU_DEBUG_OUPUT_VARS',
                             'configuration_value'       => "False",
                             'configuration_description' => TEXT_USU_DEBUG_OUPUT_VARS,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 13,
                             'date_added'                => 'now()',
                             'set_function'              => "tep_cfg_select_option(['True', 'False'], "
                            ),

                       array ('configuration_title'       => TITLE_USU_RESET_CACHE,
                             'configuration_key'         => 'USU_RESET_CACHE',
                             'configuration_value'       => "False",
                             'configuration_description' => TEXT_USU_RESET_CACHE,
                             'configuration_group_id'    => $this->config_group_id,
                             'sort_order'                => 14,
                             'date_added'                => 'now()',
                             'use_function'              => "tep_reset_cache_data_usu",
                             'set_function'              => "tep_cfg_select_option(['reset', 'False'], "
                            )
                       ];

      return $USU_configs;
      } // end method

    function addConfigSettings () {

      $USU_config = $this->setConfigSettings();

      foreach ($USU_config as $cfg_val) {
        $insert_option = array();
        $insert_value = array();
        foreach ($cfg_val as $key => $value) {
          $insert_option[] = $key;
          $insert_value[] = "'" . addslashes ($value) . "'";
          $options = implode (', ' , $insert_option);
          $values = implode (', ' , $insert_value);
        }
        tep_db_query ("INSERT INTO configuration ( $options ) VALUES ($values)");
      }
    } // end method

    function addHtaccessEntries () {
      $catalog_path = DIR_WS_CATALOG;
      $add = <<<EOH

## BEGIN USU URL rewrite rules ##
## USU Version: {$this->usu_version} ##
<IfModule mod_rewrite.c>
  RewriteEngine On
  RewriteBase $catalog_path
  RewriteRule ^([a-z0-9/-]+)-c-([0-9_]+).html$ index.php [NC,L,QSA]
  RewriteRule ^([a-z0-9/-]+)-m-([0-9]+).html$ index.php [NC,L,QSA]
  RewriteRule ^([a-z0-9/-]+)-p-([0-9]+).html$ product_info.php [NC,L,QSA]
  RewriteRule ^([a-z0-9/-]+)-pi-([0-9]+).html$ info.php [NC,L,QSA]
  ## For rewrite, path_rewrite
  RewriteRule ^([a-z0-9/-]+)-pr-([0-9]+).html$ ext/modules/content/reviews/write.php [NC,L,QSA]
  ## For standard, path_standard
  RewriteRule ^write.php/([a-z0-9/-]+)-pr-([0-9]+) ext/modules/content/reviews/write.php [NC,L,QSA]
</IfModule>
## END USU URL rewrite rules ##

EOH;

      if (is_writable ( DIR_FS_CATALOG . '.htaccess') ) {

        // Load file
        $contents = file_get_contents( $this->htaccess_path );

        // Check if there are rewriterules already in the file:
        if (strpos ($contents, "## BEGIN USU URL rewrite rules ##")) {
          // Check if version has changed:
          if (strpos ($contents, '## USU Version: ' . $this->usu_version . ' ##') === false) {
            // replace rules:
            $contents = preg_replace ('/\n(## BEGIN USU URL rewrite rules ##(?s).*## END USU URL rewrite rules ##)/m', '', $contents);
            $contents .= $add;
            $contents = preg_replace('~\r\n?~', "\n", $contents);
            file_put_contents( $this->htaccess_path, $contents );
          }
        } else {
          // add rules:
          $contents .= $add;
          file_put_contents( $this->htaccess_path, $contents );
        }
      }
    }

    /**
    * Set the sort order of our configuration group id
    *
    * @access public
    * @return Installer_Class - chaining
    */
    public function getMaxSort() {
      $result = tep_db_query($this->max_sort_query);
      $row = tep_db_fetch_array( $result );
      tep_db_free_result( $result );
      $sort_order = ( $row['current_max'] +1 );
      return $sort_order;
    } // end method
  }
