<?php
  /**
  *
  * ULTIMATE Seo Urls (version 3.0.1)
  *
  * @license http://www.opensource.org/licenses/gpl-2.0.php GNU Public License
  * Copyright 2020 JMC
  * Portions Copyright 2008-2009 FWR Media, 2005 Bobby Easland
  * @Id bootstrap.php
  */

  /**
  * Set up all needed parameters and settings
  *
  */
  class usu_Bootstrap {
    private static $_singleton = null;

    /**
    * Class constructor
    * @access private
    */
    private function __construct() {
    } // end constructor

    /**
    * Singleton instance of the class
    *
    * @access public
    * @return usu_Bootstrap
    */
    public static function i() {
     if ( !self::$_singleton instanceof usu_Bootstrap ) {
       self::$_singleton = new self;
     }
     return self::$_singleton;
    } // end method

    /**
    * Initiate the main bootstrapper methods, called by usu_Main::initiate()
    *
    * @param mixed $lng - instance of language / empty array
    * @see usu_Main::initiate()
    * @see usu_Bootstrap::getRequestUri()
    * @see usu_Bootstrap::setPaths()
    * @see usu_Bootstrap::getConfigConstants()
    * @see usu_Bootstrap::loadLanguageData()
    * @see usu_Bootstrap::actionMultiLanguageSupport()
    * @see usu_Bootstrap::loadCharacterConversions()
    * @see usu_Bootstrap::loadPageModules()
    * @see usu_Bootstrap::loadUriModules()
    * @see usu_Bootstrap::uriModulesParsePath()
    * @see usu_Bootstrap::cacheSystem()
    * @see usu_Bootstrap::setRegistry()
    * @see usu_Main::getVar()
    * @see usu_Main::setVar()
    * @see Usu_Validator::initiate();
    *
    * @access public
    * @return void
    */
    public function bootStrapper( $lng ) {
      $this->getRequestUri();  // Attempt to extract or replicate (Windows) REQUEST_URI ( strips seo url to e.g. de/my-great-category-c-32.html
      $this->setPaths(); // Set all paths used by USU PRO
      $this->getConfigConstants(); // Set the array of constants used in the USU PRO configuration
      usu_Main::i()->setVar( 'enabled', USU_ENABLED ); // dependent on adminInstalled()
      // No point in loading a load of functions that will not be used
      if ( usu_Main::i()->getVar( 'enabled' ) == 'false' ) {
        return usu_Main::i()->setVar( 'initiated', false );
      }
      $this->loadLanguageData( $lng ); // Extract language data from the osCommerce language class
      $this->actionMultiLanguageSupport(); // Reset the language based on the request
      $this->loadCharacterConversions(); // Dependent on language being set as final ( i.e. multi language functions must precede )
      $this->loadPageModules(); //Load the page modules
      $this->loadUriModules(); // These modules dictate the makeup of the seo url
      $this->uriModulesParsePath(); // Selects the correct uri module then parses the request adding key => value pairs to _GET as needed by the osCommerce system.
      $this->cacheSystem();
      usu_Main::i()->setVar( 'registry', Data_Registry::i() );// Initiate the registry
      usu_Main::i()->getVar( 'cache' )->retrieve(); // Retrieve the cached data
      usu_Main::i()->setVar( 'initiated', true );
      Usu_Validator::i()->initiate();
    } // end method

    /**
    * Attempt to extract or replicate (Windows) REQUEST_URI ( strips seo url to e.g. de/my-great-category-c-32.html )
    *
    * @uses array_key_exists()
    * @uses strlen()
    * @uses strpos()
    * @uses str_replace()
    * @uses trim()
    *
    * @access public
    * @return void
    */

    public function getRequestUri() {
      global $PHP_SELF;
      $rawpath = '';
      $finduri = new ArrayIterator( array( 'HTTP_X_ORIGINAL_URL', 'HTTP_X_REWRITE_URL', 'REQUEST_URI', 'ORIG_PATH_INFO', ) );

      while ( $finduri->valid() ) {
        if ( array_key_exists(  $finduri->current(), $_SERVER ) && ( strlen( $_SERVER[$finduri->current()] ) > 0 ) ) {
          $original_request_uri = $_SERVER[$finduri->current()];
          // If there is querystring present then remove it
          if ( false !== strpos( $original_request_uri, '?' ) ) {
            $original_request_uri = substr_replace( $original_request_uri, '', strpos( $original_request_uri, '?' ), strlen( $original_request_uri ) );
          }
          // set the original_request_uri adding back ( if it was there ) the querystring
          $querystring = $this->getRequestQueryString();
          usu_Main::i()->setVar( 'original_request_uri', ( $original_request_uri . ( tep_not_null( $querystring ) ? ( '?' . $querystring ) : '' ) ) );
          break;
        }
        $finduri->next();
      }

      $rawpath = $original_request_uri;
//      if ( DIR_WS_CATALOG != '/' && ( false !== strpos( $rawpath, DIR_WS_CATALOG ) ) ) { // Remove the DIR_WS_CATALOG path
        $rawpath = str_replace( DIR_WS_CATALOG, '/', $rawpath );
//      }
     // $rawpath = trim(str_replace( usu_Main::i()->getVar( 'filename' ), '', $rawpath ) , '/' );
      $rawpath = str_replace( $PHP_SELF, '', $rawpath );
      $rawpath = trim($rawpath , '/' ); // Remove the filename and any remaining / from left or right

      usu_Main::i()->setVar( 'request_uri', $rawpath );

    } // End Method

    private function getRequestQueryString() {
      if ( false !== ( $qs = usu_Main::i()->getVar( 'request_querystring' ) ) ) {
        return $qs;
      }
      $qs = '';
      $get_array = ( strlen( $_SERVER['QUERY_STRING'] ) > 0 ) ? explode( '&', $_SERVER['QUERY_STRING'] ) : false;
      if ( false !== $get_array ) {
        foreach ( $get_array as $index => $stringpair ) {
          if ( false === strpos( $stringpair, tep_session_name() ) ) { // Leave out the ceid
            $pair = explode('=', $stringpair);
            if ( count( $pair ) == 2 ) {
              $stringpair =  usu_cleanse( $pair[0] ) . '=' . usu_cleanse( $pair[1] );
            } else {
              $stringpair =  usu_cleanse( $stringpair );
            }
            $qs .= '&' . $stringpair;
          }
        }
      }
      usu_Main::i()->setVar( 'request_querystring', ltrim( $qs, '&' ) );
      return usu_Main::i()->getVar( 'request_querystring' );
    } // end method

    /**
    * Set all paths used by USU PRO
    * @see usu_Main::setVar()
    * @see usu_Bootstrap::getRealPath()
    *
    * @access private
    * @return void
    */
    private function setPaths() {
      $real_path = $this->getRealPath();
      $usu_path  = $real_path . 'includes/apps/ultimate_seo_urls/';
      usu_Main::i()->setVar( 'real_path', $real_path )
                   ->setVar( 'usu_path', $usu_path )
                   ->setVar( 'db_install_path', $usu_path . 'installer/' )
                   ->setVar( 'abstracts_path', $usu_path . 'abstracts/' )
                   ->setVar( 'includes_path', $usu_path . 'includes/' )
                   ->setVar( 'cache_system_path', $usu_path . 'cache_system/' )
                   ->setVar( 'cache_path', $usu_path . 'cache_system/file/' )
                   ->setVar( 'interfaces_path', $usu_path . 'interfaces/' )
                   ->setVar( 'page_modules_path', $usu_path . 'page_modules/' )
                   ->setVar( 'uri_modules_path', $usu_path . 'uri_modules/' )
                   ->setVar( 'base_url', HTTP_SERVER . DIR_WS_CATALOG );

    } // end method

    /**
    * A number of osC users don't have a full path in DIR_FS_CATALOG which was causing the caching to fail so sadly we have to do this to find the full path.
    *
    * @access private
    * @return string - path to shop root
    */
    private function getRealPath() {
      $remove_from_path = '/includes/apps/ultimate_seo_urls/main';
      $realpath = '';

      $realpath = realpath( dirname( __FILE__ ) . '/../../../../' ) . '/';
      if( file_exists( $realpath . 'product_info.php' ) && !empty( $realpath ) ) {
        return $realpath;
      }

      $realpath = str_replace( array( DIRECTORY_SEPARATOR, $remove_from_path ), array( '/', '' ), getcwd() ) . '/';
      if( file_exists( $realpath . 'product_info.php' ) && !empty( $realpath ) ) {
        return $realpath;
      }

      if ( file_exists( DIR_FS_CATALOG . 'product_info.php' ) ) {
        return DIR_FS_CATALOG;
      }
      trigger_error( 'Usu cannot find the full filepath, please ensure that DIR_FS_CATALOG in configure.php contains a FULL path', E_USER_WARNING );
    } // end method

    /**
    * Set the array of constants used in the USU PRO configuration
    *
    * @see usu_Main::setVar()
    *
    * @access private
    * @return void
    */
    private function getConfigConstants() {
      $constants = array( 'USU_RESET_CACHE'=> USU_RESET_CACHE,
                          'USU_ENABLED'=>USU_ENABLED,
                          'USU_CACHE_ON'=>USU_CACHE_ON,
                          'USU_URLS_TYPE'=>USU_URLS_TYPE,
                          'USU_FILTER_SHORT_WORDS'=>USU_FILTER_SHORT_WORDS,
                          'USU_REMOVE_ALL_SPEC_CHARS'=>USU_REMOVE_ALL_SPEC_CHARS,
                          'USU_CACHE_DAYS'=>USU_CACHE_DAYS,
                          'USU_OUPUT_PERFORMANCE'=>USU_OUPUT_PERFORMANCE,
                          'USU_ADD_CAT_PARENT'=>USU_ADD_CAT_PARENT,
                          'USU_DEBUG_OUPUT_VARS'=>USU_DEBUG_OUPUT_VARS,
                          'USU_CACHE_SYSTEM'=>USU_CACHE_SYSTEM,
                          'USU_PRODUCTS_LINK_TEXT_ORDER'=>USU_PRODUCTS_LINK_TEXT_ORDER,
                        );

      usu_Main::i()->setVar( 'config_settings', $constants );
    } // End method


    /**
    * Load the language based character conversions
    *
    * If there is a file based character conversion we use that, if not we fall back on the old admin settings
    * @uses is_readable()
    * @uses is_array()
    * @uses defined()
    * @uses explode()
    * @uses trim()
    * @see usu_Main::getVar()
    * @see usu_Main::setVar()
    *
    * @access private
    * @return void
    */
    private function loadCharacterConversions() {
      // Check to see if there is a character conversion language file before trying the admin settings
      $character_conv_filepath = usu_Main::i()->getVar( 'includes_path' ) . 'character_conversion/' . usu_Main::i()->getVar( 'language' ) . '.php';
      if ( is_readable( $character_conv_filepath ) ) {
        include_once $character_conv_filepath;
        if ( isset( $char_convert ) && is_array( $char_convert ) && !empty( $char_convert ) ) {
          return usu_Main::i()->setVar( 'character_conversion', $char_convert );
        }
      }
      usu_Main::i()->setVar( 'character_conversion', false );
    } // End method

    /**
    * Load the page modules
    *
    * Modules allow USU to deal with the different requests and added contributions
    * @example categories, products, manufacturers - contributions may be articles or information pages
    * Module filenames are the same as the page they represent e.g. module for index.php is named index.php
    * @uses str_replace()
    * @uses substr()
    * @uses is_readable()
    * @uses call_user_func()
    *
    * @access private
    * @return void
    */
    private function loadPageModules() {
      include_once usu_Main::i()->getVar( 'abstracts_path' ) . 'page_modules.php';
      $modules = usu_dir_iterator( usu_Main::i()->getVar( 'page_modules_path' ) );
      $page_modules = array();
      foreach ( $modules as $index => $module ) {
        $basename = str_replace( '.php', '', $module );
        $class = module_naming_convention( $basename, '_Page_Module' );
        if ( ( substr( $module, -4, 4 ) == '.php' ) && ( is_readable(  usu_Main::i()->getVar( 'page_modules_path' ) . $module ) ) ) {
          include_once usu_Main::i()->getVar( 'page_modules_path' ) . $module;
          $page_modules[$basename] = call_user_func( array( $class, 'i' ) );
        }
      }
      return usu_Main::i()->setVar( 'page_modules', $page_modules );
    } // End method

    /**
    * Load uri modules
    *
    * These modules dictate the makeup of the seo url
    * @example rewrite, path_rewrite, standard, path_standard
    * @uses is_readable()
    * @uses str_replace()
    * @uses substr()
    * @see usu_Main::getVar()
    * @see usu_Main::setVar()
    * @see includes/usu_general_functions.php - module_naming_convention()
    *
    * @access private
    * @return void
    */
    private function loadUriModules() {
      include_once usu_Main::i()->getVar( 'abstracts_path' ) . 'uri_modules.php';
      $modules = usu_dir_iterator( usu_Main::i()->getVar( 'uri_modules_path' ) );
      $uri_modules = array();
      foreach ( $modules as $index => $module ) {
        $basename = str_replace( '.php', '', $module );
        $class = module_naming_convention( $basename, '_Uri_Module' );
        if ( ( substr( $module, -4, 4 ) == '.php' ) && ( is_readable(  usu_Main::i()->getVar( 'uri_modules_path' ) . $module ) ) ) {
          include_once usu_Main::i()->getVar( 'uri_modules_path' ) . $module;
          $uri_modules[$basename] = new $class;
        }
      }
      return usu_Main::i()->setVar( 'uri_modules', $uri_modules );
    } // End method


    /**
    * Selects the correct uri module then parses the request adding key => value pairs to _GET as needed by the osCommerce system.
    * @uses is_readable()
    * @uses substr()
    *
    * @access private
    * @return void
    */
    private function uriModulesParsePath() {
      if ( !is_readable( usu_Main::i()->getVar( 'page_modules_path' ) . usu_Main::i()->getVar( 'filename' ) ) ) {
        return false;
      }
      $uri_modules = usu_Main::i()->getVar( 'uri_modules' );
      foreach ( $uri_modules as $object ) {
        if ( false !== ( $object->isValidUri() ) ) {
          $object->parsePath();
        }
      }
    } // End method

    /**
    * Extract language data from the osCommerce language class
    *
    * This is used for the multi language support
    *
    * @param mixed $lng - instance of language or empty array
    *
    * @access private
    * @return void
    */
    public function loadLanguageData( $lng ) {
      $current_language = '';
      $catalog_languages = array();

      if ( !empty( $lng ) && ( $lng instanceof language ) ) {
        $catalog_languages = $lng->catalog_languages;
        foreach ( $catalog_languages as $code => $detail ) {
          if ( $detail['directory'] == usu_Main::i()->getVar( 'language' ) ) {
            $current_code = $code;
          }
        }
        $current_language = $lng->language;
      } else {
        $languages_query = tep_db_query( "SELECT * FROM languages ORDER BY sort_order" );
        $current_language = '';
        while ( $languages = tep_db_fetch_array( $languages_query ) ) {
          $catalog_languages[$languages['code']] = array( 'id' => $languages['languages_id'],
                                                          'name' => $languages['name'],
                                                          'image' => $languages['image'],
                                                          'directory' => $languages['directory'] );
          if ( $languages['directory'] == usu_Main::i()->getVar( 'language' ) ) {
            $current_language =  $catalog_languages[$languages['code']];
            $current_code = $languages['code'];
          }
        }
      }
      $current_language =  array_merge( (array)$current_language, array( 'code' => $current_code ) );
      usu_Main::i()->setVar( 'catalog_languages', $catalog_languages )
                   ->setVar( 'current_language', $current_language );
    } // End method

    /**
    * Reset the language based on the request
    *
    * This is the core of the multi language system.
    *
    * @access private
    * @return void
    */
    private function actionMultiLanguageSupport() {
      // Exit if Multi language support is not required

      $catalog_languages = usu_Main::i()->getVar( 'catalog_languages' );
      if (count ( $catalog_languages ) < 2  || isset( $_GET['language'] )) {
        return false;
      }

      $possible_lang = false;

      // Looks for "es/" , "en/", "de/" at the begining of the string
      preg_match( '@^[a-z]{2}/@', usu_Main::i()->getVar( 'request_uri' ), $matches ); // Try to find language (ie. de/) at the front of the request

      if ( strlen( usu_Main::i()->getVar( 'request_uri' ) ) == 2 ) { // Possibly a language name ( like de ) if two characters
        $possible_lang = usu_Main::i()->getVar( 'request_uri' );
      } elseif ( !empty( $matches ) && ctype_alpha( rtrim( $matches[0], '/' ) ) ) { // did we get a regex match?
        $possible_lang = rtrim( $matches[0], '/' );
      }

      if ( false !== $possible_lang ) { // We have some sort of match so iterate through the language array
        foreach( $catalog_languages as $lang_code => $lang_detail ) {
          if ( $possible_lang == $lang_code ) { // We have a match
            usu_Main::i()->setVar( 'languages_id', $lang_detail['id'] ); // Reset $languages_id to the new value
            usu_Main::i()->setVar( 'language', $lang_detail['directory'] ); // Reset $language to the new value
            $this->loadLanguageData( array() );
            return; // Job done so exit the method
          }
        }
      }

      $current_language = usu_Main::i()->getVar( 'current_language' );

      // We have no language markers to use so we really should be using the default language
      if ( DEFAULT_LANGUAGE != $current_language['code'] ) { // We are not using the default language so need to change to it
        usu_Main::i()->setVar( 'languages_id', $catalog_languages[DEFAULT_LANGUAGE]['id'] ); // Reset $languages_id to the new value
        usu_Main::i()->setVar( 'language', $catalog_languages[DEFAULT_LANGUAGE]['directory'] ); // Reset $language to the new value
        $this->loadLanguageData( array() );
      }
    } // End method

    /**
    * Load the chosen cache system
    * @uses is_readable()
    * @uses call_user_func()
    * @uses trigger_error()
    * @see usu_Main::getVar()
    * @see usu_Main::setVar()
    *
    * @throws - triggers an error of type E_USER_WARNING if the cache system cannot be found
    * @access private
    * @return void
    */
    private function cacheSystem() {
      include_once usu_Main::i()->getVar( 'interfaces_path' ) . 'cache_interface.php';
      if ( is_readable( usu_Main::i()->getVar( 'cache_system_path' ) . USU_CACHE_SYSTEM . '.php' ) ) {
        include_once usu_Main::i()->getVar( 'cache_system_path' ) . USU_CACHE_SYSTEM . '.php';
        $class_name = module_naming_convention( USU_CACHE_SYSTEM, '_Cache_Module' );
        usu_Main::$performance['cache_system'] = $class_name;
        $cache_object = call_user_func( array( $class_name, 'i' ) );
        return usu_Main::i()->setVar( 'cache', $cache_object );
      }
      trigger_error( 'Can not find cache system: <b>' . USU_CACHE_SYSTEM . '</b>', E_USER_WARNING );
    } // End method
  } // End class
