<?php
  /**
  *
  * ULTIMATE Seo Urls (version 3.0.1)
  *
  * @license http://www.opensource.org/licenses/gpl-2.0.php GNU Public License
  * Copyright 2020 JMC
  * Portions Copyright 2008-2009 FWR Media, 2005 Bobby Easland
  * @Id usu5.php
  */

 /**
 * Load the general reusable functions
 */
  require_once 'includes/apps/ultimate_seo_urls/includes/usu_general_functions.php';
  /**
  * @see usu_Bootstrap
  */
  require_once 'includes/apps/ultimate_seo_urls/main/bootstrap.php';
  /**
  * @see Usu_Validator
  */
  require_once 'includes/apps/ultimate_seo_urls/main/validator.php';
  /**
  * @see Data_Registry
  */
  require_once 'includes/apps/ultimate_seo_urls/main/registry.php';
  /**
  * @see hreflang
  */
  require_once 'includes/apps/ultimate_seo_urls/includes/hreflang.php';


  /**
  * Main USU PRO class
  *
  */
  class usu_Main {

    private static $_singleton = null;
    private $vars = array();
    public static $usu_version = '3.0.1';

    // Store performance data
    public static $performance = array( 'seo_urls'              => 0,
                                        'seo_url_array'         => array(),
                                        'std_urls'              => 0,
                                        'std_url_array'         => array(),
                                        'queries_saved'         => 0,
                                        'querycount'            => 0,
                                        'queries'               => array(),
                                        'time_extracting_cache' => 0,
                                        'cache_loaded'          => 'False' );
    /**
    * Class constructor
    * @access private
    */
    private function __construct() {
    } // end constructor

    /**
    * Class destructor
    *
    * Stores the cache and outputs and performance data requested
    *
    * @access public
    * @return void
    */
    public function __destruct() {
      if ( defined( 'USU_ENABLED' ) && ( USU_ENABLED == 'True' ) &&( USU_CACHE_ON == 'True') ) {
        $this->getVar( 'cache' )->store( $this->getVar( 'registry' )->store() );
      }
      if ( defined( 'USU_OUPUT_PERFORMANCE' ) && ( USU_OUPUT_PERFORMANCE == 'True' ) ) {
        performance();
      }
      if ( defined( 'USU_DEBUG_OUPUT_VARS' ) && ( USU_DEBUG_OUPUT_VARS == 'True' ) ) {
        usu_show_vars( $this->vars );
      }
    } // end destructor

    /**
    * Return a singleton instance of the class
    *
    * @access public
    * @return usu_Main
    */
    public static function i() {
      if ( !self::$_singleton instanceof usu_Main ) {
        self::$_singleton = new self;
      }
     return self::$_singleton;
    } // end method

    /**
    * Actioned on includes/application_top.php
    *
    * After the variables have all been set this is were USU is actually initiated
    *
    * @param mixed $lng - instance of language / empty array
    * @param int $languages_id - passed by reference as USU may reset this value in multi language mode
    * @param string $language - passed by reference as USU may reset this value in multi language mode
    * @param $force_language - force links to be produced for a particular language
    * @see usu_Bootstrap::bootStrapper()
    *
    * @access public
    * @return void
    */
    public function initiate( $lng, &$languages_id, &$language, $force_language = false ) {
      if ( false !== $force_language ) {
        $this->vars['languages_id'] = $languages_id;
        $this->vars['language'] = $language;
        return usu_Bootstrap::i()->loadLanguageData( array() );
      }
      usu_Bootstrap::i()->bootStrapper( $lng );
      $languages_id = $this->vars['languages_id'];
      $language = $this->vars['language'];
    } // end method

    /**
    * USU link wrapper, defaults to the standard osCommerce tep_href_link() wrapper if not a valid seo url request
    *
    * @see includes/usu_general_functions.php - tep_href_link_original() - this is the standard osC tep_href_link() wrapper
    * @see usu_Main::getVar()
    * @see usu_Main::monitorPerformance()
    * @uses str_replace()
    * @uses substr()
    * @uses strlen()
    * @uses array_key_exists()
    * @uses preg_match()
    * @uses is_readable()
    *
    * @param string $page - base filename or sometimes a path ( e.g. ext/modules/payment/paypal.php )
    * @param string $parameters - querystring parameters
    * @param string $connection - SSL / NONSSL
    * @param bool $add_session_id
    * @param bool $search_engine_safe
    *
    * @access public
    * @return string - url
    */
    public function hrefLink( $page, $parameters, $connection, $add_session_id, $search_engine_safe ) {
     // Badly coded shops often pass in odd characters
     $parameters = str_replace( array( '&amp;', 'amp;' ), array( '&', '', ), $parameters );

     // Page can be stored on any subdirectory:
     $page_module_name = pathinfo($page)['filename'];
     $this->vars['page_module_name'] = $page_module_name;
     // 1001 reasons to revert to using the standard osCommerce link wrapper

     if ( !array_key_exists( 'initiated', $this->vars )
            || ( false === $this->vars['initiated'] ) // USU not yet fully initiated
            || !preg_match( '@^[a-z0-9_]+\.php$@i', $this->vars['filename'] ) // Not a valid filename
// This blocks links from being generated inside other pages than the ones with page_module
//            || !is_readable( $this->getVar( 'page_modules_path' ) . $this->vars['filename'] ) // No module for this file
            || ( USU_ENABLED == 'False' ) // USU is turned off
            || !tep_not_null( $parameters ) // All Seo Urls require parameters
            || !array_key_exists( 'page_modules', $this->vars ) // Page module objects not loaded
            || ( false === $this->vars['page_modules'] ) // Page module objects not loaded
            || !array_key_exists( $page_module_name, $this->vars['page_modules'] ) // No module loaded for this page
     ) {
       return tep_href_link_original( $page, $parameters, $connection, $add_session_id, $search_engine_safe );
     }

     if ( false === ( $url = $this->vars['page_modules'][$page_module_name]->buildLink( $page, $parameters, $add_session_id, $connection ) ) ) {
       // The module returned false so we fall back on the standard osCommerce link wrapper
       return tep_href_link_original( $page, $parameters, $connection, $add_session_id, $search_engine_safe );
     }
     if ( self::monitorPerformance() ) {
       self::$performance['seo_urls']++;
       self::$performance['seo_url_array'][] = $url;
     }
     return $url;
    } // end method

    /**
    * Set the class properties out of scope
    *
    * @param mixed $var_name - variable key
    * @param mixed $value - variable value
    *
    * @access public
    * @return usu_Main - chaining
    */
    public function setVar( $var_name, $value ) {
     $this->vars[$var_name] = $value;
     return $this;
    } // end method

    /**
    * Retrieve class properties from out of scope
    *
    * @uses array_key_exists()
    * @uses is_array()
    * @param string $var_name - variable key
    * @param string $additional_key - $variable[$var_name][$additional_key]
    *
    * @access public
    * @return mixed bool false / data found by the retrieval
    */
    public function getVar( $var_name, $additional_key = false ) {

     if ( array_key_exists( $var_name, $this->vars ) ) {
       if ( ( false !== $additional_key ) && is_array( $this->vars[$var_name] ) && array_key_exists( $additional_key, $this->vars[$var_name] ) ) {
         return $this->vars[$var_name][$additional_key];
       }

       return $this->vars[$var_name];
     }
     return false;
    }  // end method

    /**
    * Query wrapper
    *
    * Purely exists to allow monitoring of the number and content of queries
    *
    * @uses microtime()
    * @uses number_format()
    * @see usu_Main::monitorPerformance()
    * @param string $sql - The query
    *
    * @access public
    * @return resource - query result
    */
    public function query( $sql ) {
      $time = microtime( true );
      $result = tep_db_query( $sql );
      $end_time = number_format ( ( microtime( true ) - $time ), 4 );
      if ( self::monitorPerformance() ) {
        self::$performance['querycount']++;
        self::$performance['queries'][] = array( 'time' => $end_time, 'query' => $sql );
      }
      return $result;
    }  // End method

    /**
    * Factory to extract data from the various cache strategies
    *
    * @see usu_Main::getVar()
    * @see Data_Registry::load()
    * @uses microtime()
    * @uses str_replace()
    * @uses number_format()
    * @uses strlen()
    * @uses gzinflate()
    * @uses base64_decode()
    * @uses unserialize()
    * @uses is_readable()
    * @uses file_get_contents()
    * @param string $cache_name - cache name to extract
    * @param string $cache_type - e.g. mysql, file etc.
    * @param object $object  iCache_System
    *
    * @access public
    * @return bool
    */
    public function extractCacheData( $cache_name, $cache_type, iCache_System $object ) {
      $cache_seconds = ( (int)USU_CACHE_DAYS * 24 * 60 * 60 );

      $timestart = microtime( true );

      switch ( true ) {
        case $cache_type == 'mysql':
          $query = str_replace( ':cache_name', $cache_name, $object->extract_query );
          $result = $this->query( $query );
          $row = tep_db_fetch_array( $result );
          tep_db_free_result( $result );
            if (!$row) {
            self::$performance['time_extracting_cache'] = number_format ( ( microtime( true ) - $timestart ), 4 );
            // return insert = true as there is nothing in the database
            return true;
          }
          if (time() > ( strtotime( $row['cache_date'] ) + $cache_seconds ) ) { // If the cache has expired
          //  $object->gc(); // Clear the cache
          // FIX: Delete only this row:
          $query = str_replace( ':cache_name', $cache_name, $object->delete_query );
          $result = $this->query( $query );

            self::$performance['time_extracting_cache'] = number_format ( ( microtime( true ) - $timestart ), 4 );
            // return insert = true as there is nothing in the database
            return true;
          } else { // the cache hasn't expired so we retrieve it and set the registry
            $cache_file_size = number_format( strlen( $row['cache_data'] ) / 1024, 2 ) . ' kb';
            $rawdata = gzinflate( base64_decode( $row['cache_data'] ) );
            $this->getVar( 'registry' )
                 ->load( unserialize( $rawdata ) );
            self::$performance['time_extracting_cache'] = number_format ( ( microtime( true ) - $timestart ), 4 );
            self::$performance['cache_loaded'] = 'True';
            // return insert = false as we have extracted data from the database
            return false;
          }
          break;
        case $cache_type == 'file':
          if ( is_readable( $this->getVar( 'cache_path' ) . $cache_name . '.cache' ) ) {
            $this->getVar( 'registry' )
                 ->load( unserialize( gzinflate( file_get_contents( $this
                 ->getVar( 'cache_path' ) . $cache_name . '.cache' ) ) ) );
            self::$performance['cache_loaded'] = 'True';
            self::$performance['time_extracting_cache'] = number_format ( ( microtime( true ) - $timestart ), 4 );
            return false;
          } else {
            self::$performance['time_extracting_cache'] = number_format ( ( microtime( true ) - $timestart ), 4 );
            return true;
          }
          break;

        case $cache_type == 'sqlite':
          $query = str_replace( ':cache_name', $cache_name, $object->extract_query );
          $result = $object->getDb()->query( $query );
          $row = $result->fetchArray();
          if ( empty( $row ) ) {
            self::$performance['time_extracting_cache'] = number_format ( ( microtime( true ) - $timestart ), 4 );
            // return insert = true as there is nothing in the database
            return true;
          }
          if ( time() > ( strtotime( $row['cache_date'] ) + $cache_seconds ) ) { // If the cache has expired
            $object->gc(); // Clear the cache
            self::$performance['time_extracting_cache'] = number_format ( ( microtime( true ) - $timestart ), 4 );
            // return insert = true as there is nothing in the database
            return true;
          } else { // the cache hasn't expired so we retrieve it and set the registry
            $cache_file_size = number_format( strlen( $row['cache_data'] ) / 1024, 2 ) . ' kb';
            $rawdata = gzinflate( base64_decode( $row['cache_data'] ) );
            $this->getVar( 'registry' )
                 ->load( unserialize( $rawdata ) );
            self::$performance['time_extracting_cache'] = number_format ( ( microtime( true ) - $timestart ), 4 );
            self::$performance['cache_loaded'] = 'True';
            // return insert = false as we have extracted data from the database
            return false;
          }
          break;
        default:
          trigger_error( __CLASS__ . '::' . __FUNCTION__ . ' unknown type passed as: ' . $type, E_USER_WARNING );
          break;
      }
    }
    /**
    * If monitoring is turned off it saves memory as the arrays are left empty
    *
    * @uses defined()
    *
    * @access public
    * @return bool
    */
    public static function monitorPerformance() {
      if ( defined( 'USU_OUPUT_PERFORMANCE' ) && USU_OUPUT_PERFORMANCE == 'True' ) {
        return true;
      }
      return false;
    } // end method

  } // end class
