<?php
  /**
  *
  * ULTIMATE Seo Urls (version 3.0.1)
  *
  * @license http://www.opensource.org/licenses/gpl-2.0.php GNU Public License
  * Copyright 2020 JMC
  * Portions Copyright 2008-2009 FWR Media, 2005 Bobby Easland
  * @Id product_info.php
  */

  /**
  * Page module for product_info
  *
  */
  class Product_Info_Page_Module extends aPage_Modules {

    private static $_singleton = null;
    /**
    * Array of _GET key dependencies for this page
    * - marker is the seo url replacement for the _GET key ( e.g. -c-1 replaces cPath=1 )
    * - query is the query used to extract the link text from the database
    * - to_replace is an array of texts that are to be replace with real values in the query
    *
    * @var array $dependencies
    */
    protected $dependencies = array( 'products_id' => array( 'marker'     => '-p-',
                                                             'query'      => "SELECT pd.products_name, m.manufacturers_name, cd.categories_name, p.products_model, p2c.categories_id
                                                                              FROM products_description pd
                                                                              INNER JOIN products_to_categories p2c ON p2c.products_id = pd.products_id
                                                                              INNER JOIN products p ON pd.products_id = p.products_id
                                                                              LEFT JOIN manufacturers m ON m.manufacturers_id = p.manufacturers_id
                                                                              INNER JOIN categories_description cd ON p2c.categories_id = cd.categories_id AND cd.language_id=':languages_id'
                                                                              WHERE pd.products_id=':pid' AND pd.language_id=':languages_id' LIMIT 1",
                                                             'to_replace' => array( ':languages_id', ':pid' ) ) );
    /**
    * The current dependency key extracted from the incoming parameters
    *
    * @var string $key - dependency key
    */
    protected $key = null;
    /**
    * extracts the key => value pairs from the querystring in order to build a unique cache name for the loaded page
    *
    * @var array $cache_name_builder
    */
    protected $cache_name_builder = array( 'products_id' => 1, 'page' => 1 );
    /**
    * Class constructor
    * @access private
    */
    private function __construct() {
    } // end constructor
    /**
    * Returns a singleton instance of this object
    *
    * @access public
    * @return Product_Info_Page_Module
    */
    public static function i() {
     if ( !self::$_singleton instanceof self ) {
       self::$_singleton = new self;
     }
     return self::$_singleton;
    } // end method
    /**
    * Retrieve the dependencies array for this page module
    *
    * @access public
    * @return array $dependencies
    */
    public function retrieveDependencies() {
      return $this->dependencies;
    }
    /**
    * Acquire an array of single or multiple link texts from the query
    * this will be cached for later retrieval.
    *
    * @see usu_Main::query()
    * @uses trim()
    *
    * @access protected
    * @return array array of link test
    */
   protected function acquireLinkText() {
    // BOF fixes sitemaps multi language url
    if (($_SERVER['REQUEST_URI'] == "/sitemaps/" || $_SERVER['REQUEST_URI'] == "/sitemaps/index.php")) {
      if (  false !== $final_text_array = usu_Main::i()->getVar( 'registry' )->retrieve( $this->key, $this->keys_index[$this->key] ) ) {
        return $final_text_array; // Returns cached value and exits
      }
    }
    // EOF fixes sitemaps multi language url

    if ( usu_Main::monitorPerformance() ) {
      usu_Main::$performance['queries_saved']++;
    }

    $result = usu_Main::i()->query( $this->query );
    $text_array = tep_db_fetch_array( $result );

    tep_db_free_result( $result );

    if ( is_null($text_array) || false === $text_array ) {
      return false;
    }

    $final_text_array = array();
    foreach ( $text_array as $key => $text ) {
      if ( tep_not_null( trim( $text ) ) ) {
        $final_text_array[$key] = $text;
      }
    }
    // We will cache this result
    usu_Main::i()->getVar( 'registry' )->attach( $this->key, $this->keys_index[$this->key], $final_text_array );
    return $final_text_array;
  }

    /**
    * The main method of this class that receives input needed to build a link
    * then finally returns a fully built seo link if it has not previousluy returned false.
    *
    * @see usu_Main::getVar()
    * @see usu_Main::setVar()
    * @see aPage_Modules::stripPathToLastNumber()
    * @see aPage_Modules::setQuery()
    * @see aPage_Modules::unsetProperties()
    * @see aPage_Modules::getDependencyKey()
    * @see aPage_Modules::setAllParams()
    * @see aPage_Modules::validRequest()
    * @see aPage_Modules::returnFinalLink()
    * @param string $page - valid osCommerce page name
    * @param string $parameters - querystring parameters
    * @param bool $add_session_id - true / false
    * @param string $connection - NONSSL / SSL
    * @param array $extract - array of _GET keys to remove from the querystring or bool false to do nothing
    * @uses trigger_error()
    * @throws - triggers an error of type E_USER_WARNING for an incorrect or inexistant dependency key
    * @access public
    * @return bool false - forces the system to return the standard osCommerce link wrapper
    * @return string - fully built seo url
    */
    public function buildLink( $page, $parameters, $add_session_id, $connection ) {
      $extract =  array( 'manufacturers_id'); // removes manufacturers_id

      $this->setAllParams( $page, $parameters, $add_session_id, $connection, $extract );
      if ( false === $this->validRequest() ) {
        $this->unsetProperties();
        return false;
      }
      $this->key = $this->getDependencyKey();
      /**
      * If the shop has issues it may pass in null values, in this case return false to force the standard osCommerce link wrapper
      */
      if ( !array_key_exists( $this->key, $this->keys_index ) || !tep_not_null( $this->keys_index[$this->key] ) ) {
        return false;
      }
      // Switch statement where the correct query and query marker replacements to use are selected via the _GET key detected
      switch ( true ) {
        case $this->key == 'products_id':
          // This array contains replacements for the to_replace array
          $this->setQuery( array( usu_Main::i()->getVar( 'languages_id' ), $this->keys_index[$this->key] ) );
          break;
        default:
          trigger_error( __CLASS__ . '::' . __FUNCTION__ . ' Incorrect or inexistant dependency key.', E_USER_WARNING );
          break;
      } // end switch
      $link_text = $this->acquireLinkText();
      // If the query returned no results then we return false forcing the use of the standard osCommerce link wrapper
      usu_Main::i()->setVar( 'page_not_found', false );
      if ( false === $link_text ) {
        usu_Main::i()->setVar( 'page_not_found', true );
        $this->unsetProperties();
        return;
      }
      // Return a fully built seo url
      return $this->returnFinalLink( usu_Main::i()
                  ->getVar( 'uri_modules', USU_URLS_TYPE )
                  ->createLinkString( $this->page, usu_Main::i()
                  ->getVar( 'uri_modules', USU_URLS_TYPE )
                  ->separateUriText( $this->linktext( $this->linkTextOrder( $link_text ) ) ), $this->dependencies[$this->key]['marker'], $this->keys_index[$this->key] ) );
    } // end method
    /**
    * Like text options offered by b,c,m,p
    *
    * Builds the link text based on the settings chosen in admin .. products name must be present or the script adds it regardless.
    *
    * @uses array_key_exists()
    * @uses stripos()
    * @uses str_split()
    * @uses array_flip()
    * @uses array_intersect_key()
    * @uses count()
    * @param array $details - link text key=>value pairs
    *
    * @access private
    * @return array $to_separate - array of link text options in the correct order
    */
    private function linkTextOrder( array $details = array() ) {
// ****************************
      $text_types = array( 'p' => $details['products_name'], 'c' => $details['categories_name'] );

     // Patch for products_model in case it doesn't exists (it's not forced in product creation):
      array_key_exists( 'products_model', $details ) ? $text_types['m'] = $details['products_model'] : null;

      // manufacturers_name is gained through a left join and may not exist
      array_key_exists( 'manufacturers_name', $details ) ? $text_types['b'] = $details['manufacturers_name'] : null;
      // Products name MUST be present, if not we force it to the end
      if ( false === stripos( USU_PRODUCTS_LINK_TEXT_ORDER, 'p' ) ) {
         $admin_order = str_split( USU_PRODUCTS_LINK_TEXT_ORDER . 'p' );
      } else $admin_order = str_split( USU_PRODUCTS_LINK_TEXT_ORDER );
      // Split the string value entered in admin ( like mcp etc ) to an array
      $text_order = array_flip( $admin_order ); // Flip key => value to value => key
      $text_to_format = array_intersect_key( $text_types, $text_order ); // return only those key => value pairs that match the admin selection
      $link_text_count = count( $admin_order );
      $ordered_array = array();
      for ( $i=0; $i<$link_text_count; $i++ ) {
      if ( ( array_key_exists( $admin_order[$i], $text_to_format ) && tep_not_null( $text_to_format[$admin_order[$i]] ) ) ) { // If a value is empty then we don't want to add it to the link text
          $to_separate[] = $text_to_format[$admin_order[$i]]; // Create an array where the  link text is in the same order as the admin selection
        }
      }
      return $to_separate;
    } // End method

  } // end class